/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "analyse.h"



/************************************ Analyse ****************************************/

Analyse::Analyse()
{
// 	m_ordrePourBarrer << YAHTZEE << CARRE << FULL << GRANDE_SUITE << PETITE_SUITE << BRELAN << DEUX_PAIRES << UNE_PAIRE;
	m_ordrePourBarrer << GRANDE_SUITE << YAHTZEE << CARRE << BRELAN << FULL << DEUX_PAIRES << PETITE_SUITE << UNE_PAIRE;
	m_ordrePourBarrer << UN << DEUX << TROIS << QUATRE << CINQ << SIX;
	lignesCliquables << UN << DEUX <<TROIS << QUATRE << CINQ << SIX;
	lignesCliquables << UNE_PAIRE << DEUX_PAIRES << BRELAN << CARRE << FULL << PETITE_SUITE << GRANDE_SUITE << CHANCE << YAHTZEE;

}


void Analyse::trace(QString txt) {
	if (m_joueur == PC)
	{
		ra.trace(txt);
	}
}


void Analyse::init(int joueur, int numSession,int numLigneDescendante, int numLigneMontante, QList<int> &lignesLibres, int d1, int d2, int d3, int d4, int d5, int ecartDesc, int ecartAsc, int ecartLibre)
{
	// note le joueur
	m_joueur = joueur;

	// récupère les valeurs des dés

	enCoursDescendante = numLigneDescendante;
	enCoursMontante = numLigneMontante;
	enCoursLibre = lignesLibres;
	m_ecartDescendante = ecartDesc;
	m_ecartMontante = ecartAsc;
	m_ecartLibre = ecartLibre;
	m_avancementDescendante = enCoursDescendante - UN;
	m_avancementMontante = YAHTZEE - enCoursMontante;
	numLancer = numSession;

	dicesList.clear();
	dicesList << d1 << d2 << d3 << d4 << d5;

	lignesJouables = enCoursLibre;

	if (enCoursDescendante <= YAHTZEE) lignesJouables << enCoursDescendante;

	if (enCoursMontante >= UN) lignesJouables << enCoursMontante;

	/************** RAZ des résultats **************/
	for ( int ligne = PREMIERE_LIGNE ; ligne <= DERNIERE_LIGNE ; ++ligne )
	{
		resultat[ligne][NBR] = 0;
		setPoints(ligne, 0);
	}

	resultat[CHANCE][NBR] = 1;	//toujours la chance !!

	paire1.clear();
	paire2.clear();
	deuxPaires.clear();
	brelan.clear();
	carre.clear();
	full.clear();
	petiteSuite.clear();
	grandeSuite.clear();
	yahtzee.clear();

	ra.clear();

	if (numLancer == 1)
	{
//		trace (tr("\n----------- Nouveau tour -----------"));
	}
//	trace (tr("\n ---> Héhé lancer n°%1 : %2 %3 %4 %5 %6").arg(numLancer).arg(d1).arg(d2).arg(d3).arg(d4).arg(d5));


	/************** ANALYSE **************/

	// 1 Calcul des colonnes NBR, POINTS pour les 6 lignes du haut

	// Calcul des points de la partie du haut, et détection des figures multiples (brelan, carré...
	for (int valeur = 1 ; valeur <= 6 ; valeur++)
	{
		int count = dicesList.count(valeur);
		resultat[valeur-1][NBR] = count;
		setPoints(valeur-1, count * valeur);
	}

	// on détermine les figures existantes
	// ex. : si lancer de és = 3, 5, 5, 4, 5
	// l'algorithme retourne
	// paire1 = 1,2 (dés 2 et 3)
	// paire2 vide
	// brelan = 1, 2, 4
	// carré vide
	// yahtzee vide
	for (int ligne = UN ; ligne <= SIX ; ligne++)
	{
		// recherche des dés qui ont la même valeur

		int valeur = ligne+1;
		QList<int> desConcernes = listeDesDontValeurEst(valeur);

		switch (desConcernes.count())
		{
			case 2:
				if (!paire1.exists())
				{
					paire1.set(desConcernes, true, valeur);
				}
				else
				{
					if (!paire2.exists())
					{
						paire2.set(desConcernes, true, valeur);
					}
				}
				break;

			case 3:
				brelan.set(desConcernes, true, valeur);
				break;

			case 4:
				carre.set(desConcernes, true, valeur);
				break;

			case 5:
				yahtzee.set(desConcernes, true, valeur);
				break;
		}
	}
//	 Les figures pures sont enregistrées
	// 	on attaque les figures "incluses" : dans un brelan il y a une paire, etc...

	if (brelan.exists())
	{
		if (paire1.exists())
		{
			full = paire1 + brelan;
			full.setPure(true);
			// et paire2 n'existe pas... Donc :
			paire2.set(brelan.getList().mid(0, 2), false, brelan.getValue());	// impure donc false
		}
		else
		{
			paire1.set(brelan.getList().mid(0, 2), false, brelan.getValue());
		}
	}

	if (carre.exists())
	{
		brelan.set(carre.getList().mid(0, 3), false, carre.getValue());	//arbitraire
		paire1.set(carre.getList().mid(0, 2), false, carre.getValue());	//arbitraire
	}

	if (yahtzee.exists())
	{
		full = yahtzee;
		carre.set(yahtzee.getList().mid(0, 4), false, yahtzee.getValue());
		brelan.set(yahtzee.getList().mid(0, 3), false, yahtzee.getValue());
		paire1.set(yahtzee.getList().mid(0, 2), false, yahtzee.getValue());
	}

	if (paire1.exists() && paire2.exists()) deuxPaires = paire1+paire2;

	// on prépare les totaux du bas
	if (yahtzee.exists()) resultat[YAHTZEE][NBR] = 1;

	if (carre.exists()) resultat[CARRE][NBR] = 1;

	if (brelan.exists()) resultat[BRELAN][NBR] = 1;

	if (full.exists()) resultat[FULL][NBR] = 1;

	if (deuxPaires.exists()) resultat[DEUX_PAIRES][NBR] = 1;

	if (paire1.exists()) resultat[UNE_PAIRE][NBR] = 1;

	// 2 on attaque PS, GS

	QList<int> ps;
	ps << dicesList.indexOf(1) << dicesList.indexOf(2) << dicesList.indexOf(3) << dicesList.indexOf(4);

	if (ps.contains(-1))
	{
		ps.clear();
		ps << dicesList.indexOf(2) << dicesList.indexOf(3) << dicesList.indexOf(4) << dicesList.indexOf(5);

		if (ps.contains(-1))
		{
			ps.clear();
			ps << dicesList.indexOf(3) << dicesList.indexOf(4) << dicesList.indexOf(5) << dicesList.indexOf(6);

			if (ps.contains(-1)) ps.clear();
		}
	}

	if (!ps.isEmpty())
	{
		petiteSuite.set(ps, true);
		resultat[PETITE_SUITE][NBR] = 1;
	}

	QList<int> gs;
	gs << dicesList.indexOf(1) << dicesList.indexOf(2) << dicesList.indexOf(3) << dicesList.indexOf(4) << dicesList.indexOf(5);
	if (gs.contains(-1))
	{
		gs.clear();
		gs << dicesList.indexOf(2) << dicesList.indexOf(3) << dicesList.indexOf(4) << dicesList.indexOf(5) << dicesList.indexOf(6);
		if (gs.contains(-1))
		{
			gs.clear();
		}
	}

	if (!gs.isEmpty())
	{
		grandeSuite.set(gs, true);
		resultat[GRANDE_SUITE][NBR] = 1;
	}

	// on compte les points du bas
	for ( int ligne = PREMIERE_FIGURE ; ligne <= DERNIERE_LIGNE ; ligne++ )
	{
		setPoints (ligne, valeurFigure(ligne) * resultat[ligne][NBR] );
	}

	setSuggestions();
}


int Analyse::valeurFigure( int figure )
{
	int value;
	switch ( figure )
	{
		case UNE_PAIRE:
		case DEUX_PAIRES:
		case BRELAN:
		case CARRE:
		case CHANCE:
			value = sommeDes();
			break;
		case FULL:
			value = 25;
			break;
		case PETITE_SUITE:
			value = 30;
			break;
		case GRANDE_SUITE:
			value = 40;
			break;
		case YAHTZEE:
			value = 50;
			break;
		default:
			value = 0;
			break;
	}
	return value;
}


QList<int> Analyse::getPossible()	// liste des lignes que l'on peut marquer
{
	QList<int> possibleList;

	if (enCoursDescendante <= YAHTZEE && getPoints(enCoursDescendante) > 0) {
		possibleList << enCoursDescendante;
	}

	if (enCoursMontante >= UN && getPoints(enCoursMontante) > 0) {
		possibleList << enCoursMontante;
	}

	foreach (int ligne, enCoursLibre) {
		if (getPoints(ligne) > 0) {
			possibleList << ligne;
		}
	}
	return possibleList;
}


void Analyse::setAction( int ligne ) // choisit la colonne où on doit marquer la ligne
{
	bool estDansDescendante = (enCoursDescendante == ligne);
	bool estDansMontante = (enCoursMontante == ligne);
	bool estDansLibre = (enCoursLibre.contains(ligne));
	bool estDansHaut = (ligne <= SIX);
// 	bool estDansBas = !estDansHaut;

	QList<int> candidats;
	if (estDansHaut) { // priorité à la colonne où l'écart est le plus important
		if (m_ecartDescendante > m_ecartMontante) {
			if (estDansDescendante) candidats << COLONNE_DESCENDANTE;
			if (estDansMontante) candidats << COLONNE_MONTANTE;
		}
		else {
			if (estDansMontante) candidats << COLONNE_MONTANTE;
			if (estDansDescendante) candidats << COLONNE_DESCENDANTE;
		}

		if (estDansLibre) candidats << COLONNE_LIBRE;
	}
	else { // priorité à la montante
		if (estDansMontante) candidats << COLONNE_MONTANTE;
		if (estDansDescendante) candidats << COLONNE_DESCENDANTE;
		if (estDansLibre) candidats << COLONNE_LIBRE;
	}

	int colonne = candidats[0];
	ra.setAction(ligne, colonne);
}


void Analyse::setSuggestions()
{
	bool cLeDernierTour = (numLancer == 3);

	if (!tenterFigureNonAmeliorable()) {
		if (cLeDernierTour) {
			marquer();
		}
		else {
			tenterAmeliorer();
//			if (ra.getRejouer().isEmpty()) {
//				marquer();
//			}
		}
	}
}


bool Analyse::tenterFigureNonAmeliorable()
{
	QList<int> possibleList(getPossible());

	if (yahtzee.exists()) {
		if (possibleList.contains(YAHTZEE)) {
			setAction(YAHTZEE);
			return true;
		}
		if (possibleList.contains(yahtzee.getValue()-1)) {
			setAction(yahtzee.getValue()-1);
			return true;
		}
	}

	if (full.exists()) {

		if (enCoursMontante == FULL) {
			ra.setAction(FULL, COLONNE_MONTANTE);
			return true;
		}
		if (enCoursDescendante == FULL) {
			ra.setAction(FULL, COLONNE_DESCENDANTE);
			return true;
		}

		if (possibleList.contains(brelan.getValue()-1)) {
			return false;
		}

		if (possibleList.contains(FULL)) {
			if (sommeDes() <= valeurFigure(FULL)) {
				setAction(FULL);
				return true;
			}
			if (possibleList.contains(BRELAN)) {
				setAction(BRELAN);
				return true;
			}

			if (possibleList.contains(DEUX_PAIRES)) {
				setAction(DEUX_PAIRES);
				return true;
			}

			if (possibleList.contains(UNE_PAIRE)) {
				setAction(UNE_PAIRE);
				return true;
			}

			setAction(FULL);
			return true;
		}
	}

	if (grandeSuite.exists()) {
		if (possibleList.contains(GRANDE_SUITE)) {
			setAction(GRANDE_SUITE);
			return true;
		}
		if (possibleList.contains(PETITE_SUITE)) {
			setAction(PETITE_SUITE);
			return true;
		}
	}

	if (petiteSuite.exists()) {
		if (enCoursMontante == PETITE_SUITE) {
			ra.setAction(PETITE_SUITE, COLONNE_MONTANTE);
			return true;
		}

		if (enCoursDescendante == PETITE_SUITE) {
			ra.setAction(PETITE_SUITE, COLONNE_DESCENDANTE);
			return true;
		}
	}

	return false;
}

void Analyse::tenterAmeliorer()
{
	// fabrication de la liste des lignes qui d'ores et déjà rapportent
	QList<int> lignesQuiRapportent(getPossible());

// 	bool descendanteEstDansHaut = (enCoursDescendante <= SIX);
// 	bool montanteEstDansBas = (enCoursMontante > SIX);

	if (tenterDescendante()) return;
	if (tenterMontante()) return;
	if (tenterPetiteSuite()) return;
	if (tenterCarre()) return;
	if (tenterBrelan()) return;
	if (tenterDeuxPaires()) return;
	if (tenterPaire()) return;
	if (tenterHaut2()) return;
	if (tenterFaireSuite()) return;
	if (tenterChance()) return;
	if (tenterGarderMeilleurs()) return;

	// en désespoir de cause, tout rejouer
//	trace (tr("Rejouer tous les dés"));
	setAction (QList<int>() << UN << DEUX << TROIS << QUATRE << CINQ);
	return;
}

/************************* PETITE_SUITE et GRANDE_SUITE *******************/
bool Analyse::tenterPetiteSuite() {
	if (petiteSuite.exists()) {
//		trace (tr("Il y a une PETITE_SUITE"));
		if (lignesJouables.contains(PETITE_SUITE)) {
			if (lignesJouables.contains(GRANDE_SUITE)) {
//				trace (tr("Tenter la GRANDE_SUITE"));
				setAction(toutSauf(petiteSuite));
				return true;
			}
//			trace (tr("Marquer la PETITE_SUITE"));
			setAction(PETITE_SUITE);
			return true;
		}
	}
	return false;
}

/************************* HAUT DE MONTANTE ET DESCENDANTE *******************/
bool Analyse::tenterDescendante() {
		int limiteInfRelance = numLancer == 1 ? 2 : 3;
		if (enCoursDescendante <= 3) limiteInfRelance--;
	if (getCount(enCoursDescendante) >= limiteInfRelance && !(brelan.exists() && brelan.getValue() > 3)) {
//		trace (tr("Chercher à améliorer la colonne descendante"));
		setAction(toutSauf(enCoursDescendante+1));
		return true;
	}

	return false;
}

/************************* HAUT DE MONTANTE ET DESCENDANTE *******************/
bool Analyse::tenterMontante() {
/*	int limiteInfRelance = numLancer == 1 ? 1 : 2;*/

	if (enCoursDescendante > YAHTZEE && enCoursLibre.isEmpty()) {
//		trace (tr("Chercher à améliorer la colonne montante"));
		setAction(toutSauf(enCoursMontante+1));
		return true;
	}

	if (getCount(enCoursMontante) >= 1) {
//		trace (tr("Chercher à améliorer la colonne montante"));
		setAction(toutSauf(enCoursMontante+1));
		return true;
	}
	return false;
}


/************************* CARRE *******************/
bool Analyse::tenterCarre() {
	if (carre.exists()) {
//		trace (tr("Il y a un CARRE"));
		if (lignesJouables.contains(carre.getValue()-1)) {
//			trace (tr("Tenter plutôt les lignes du haut (ou le YAHTZEE)"));
			ra.setAction(toutSauf(carre));
			return true;
		}

		if (enCoursMontante == CARRE || enCoursDescendante == CARRE) {
//			trace (tr("Tenter d'améliorer le CARRE"));
			ra.setAction(toutSauf(carre, 4));
			return true;
		}

		if (lignesJouables.contains(CARRE)) {
//			trace (tr("Tenter d'améliorer le CARRE"));
			ra.setAction(toutSauf(carre, 4));
			return true;
		}

		if (lignesJouables.contains(FULL)) {
//			trace (tr("Tenter le FULL"));
			ra.setAction(toutSauf(brelan));
			return true;
		}

		if (lignesJouables.contains(YAHTZEE)) {
//			trace (tr("Tenter le YAHTZEE"));
			ra.setAction(toutSauf(carre));
			return true;
		}
	}
	return false;
}

	/************************* BRELAN *******************/
bool Analyse::tenterBrelan() {
	if (brelan.exists()) {
//		trace (tr("Il y a un BRELAN"));

		if (lignesJouables.contains(brelan.getValue()-1)) {
//			trace (tr("Tenter plutôt les lignes du haut"));
			ra.setAction(toutSauf(brelan));
			return true;
		}

		if (enCoursMontante == BRELAN || enCoursDescendante == BRELAN) {
//			trace (tr("Tenter d'améliorer le BRELAN"));
			ra.setAction(toutSauf(brelan, 4));
			return true;
		}

		if (lignesJouables.contains(BRELAN)) {
//			trace (tr("Tenter d'améliorer le BRELAN"));
			if (brelan.getValue() > 3) {
				ra.setAction(toutSauf(brelan, 4));
			}
			else {
				setAction (garder(QList<int>() << 4 << 5 << 6));		// 11/02/09 conseils de Caro
			}
			return true;
		}

		if (lignesJouables.contains(FULL)) {
//			trace (tr("Tenter le FULL"));
			ra.setAction(toutSauf(brelan));
			return true;
		}

		if (lignesJouables.contains(CARRE)) {
//			trace (tr("Tenter le CARRE"));
			ra.setAction(toutSauf(brelan));
			return true;
		}

		if (lignesJouables.contains(YAHTZEE)) {
//			trace (tr("Tenter le YAHTZEE"));
			ra.setAction(toutSauf(brelan));
			return true;
		}

	}

	return false;
}

/************************* DEUX_PAIRES *******************/
bool Analyse::tenterDeuxPaires() {
	if (deuxPaires.exists()) {
//		trace (tr("Il y a DEUX_PAIRES"));
		if (lignesJouables.contains(paire2.getValue()-1)) {
//			trace (tr("Tenter plutôt les lignes du haut"));
			ra.setAction(toutSauf(paire2));
			return true;
		}

		if (lignesJouables.contains(paire1.getValue()-1)) {
//			trace (tr("Tenter plutôt les lignes du haut"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (enCoursMontante == DEUX_PAIRES || enCoursDescendante == DEUX_PAIRES) {
//			trace (tr("Tenter d'améliorer les DEUX_PAIRES"));
			ra.setAction(toutSauf(deuxPaires, 4));
			return true;
		}

		if (lignesJouables.contains(DEUX_PAIRES)) {
//			trace (tr("Tenter d'améliorer les DEUX_PAIRES"));
			ra.setAction(toutSauf(deuxPaires, 4));
			return true;
		}

		if (lignesJouables.contains(FULL)) {
//			trace (tr("Rejouer le 5è dé pour le FULL"));
			ra.setAction(toutSauf(deuxPaires));
			return true;
		}

		if (lignesJouables.contains(CARRE)) {
//			trace (tr("Tenter le CARRE"));
			ra.setAction(toutSauf(paire2));
			return true;
		}

		if (lignesJouables.contains(BRELAN)) {
//			trace (tr("Tenter le BRELAN"));
			ra.setAction(toutSauf(paire2));
			return true;
		}

		if (lignesJouables.contains(UNE_PAIRE)) {
//			trace (tr("Tenter d'améliorer la PAIRE"));
			ra.setAction(toutSauf(paire2, 4));
			return true;
		}

		if (lignesJouables.contains(YAHTZEE)) {
//			trace (tr("Tenter le YAHTZEE"));
			ra.setAction(toutSauf(paire2));
			return true;
		}
	}

	return false;
}

	/************************* PAIRE *******************/
bool Analyse::tenterPaire() {
	if (paire1.exists()) {
//		trace (tr("Il y a une PAIRE"));
		if (lignesJouables.contains(paire1.getValue()-1)) {
//			trace (tr("Tenter plutôt les lignes du haut"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (enCoursMontante == UNE_PAIRE || enCoursDescendante == UNE_PAIRE) {
//			trace (tr("Tenter d'améliorer la PAIRE"));
			ra.setAction(toutSauf(paire1, 4));
			return true;
		}

		if (lignesJouables.contains(FULL)) {
//			trace (tr("Tenter le FULL"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (lignesJouables.contains(BRELAN)) {
//			trace (tr("Tenter le BRELAN"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (lignesJouables.contains(CARRE)) {
//			trace (tr("Tenter le CARRE"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (lignesJouables.contains(DEUX_PAIRES)) {
//			trace (tr("Tenter les DEUX_PAIRES"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

		if (lignesJouables.contains(UNE_PAIRE)) {
//			trace (tr("Tenter d'améliorer la PAIRE"));
			ra.setAction(toutSauf(paire1, 4));
			return true;
		}

		if (lignesJouables.contains(YAHTZEE)) {
//			trace (tr("Tenter le YAHTZEE"));
			ra.setAction(toutSauf(paire1));
			return true;
		}

	}

	return false;
}

/*************** Tests de CHANCE montante et descendante ************/
bool Analyse::tenterChance() {
	if (numLancer < 2) return false;
	if (enCoursDescendante == CHANCE || enCoursMontante == CHANCE) {
//		trace (tr("Tenter un bon score à la CHANCE : on rejoue tout les valeurs < 4"));
		setAction( toutSauf(QList<int>(), 4) );
		return true;
	}

	return false;
}

/*************** On teste à nouveau le haut ************/
bool Analyse::tenterHaut2() {

	for (int ligne = UN ; ligne <= SIX ; ligne++) {
		if (getPoints(ligne) > 0 && lignesJouables.contains(ligne)) {
//			trace (tr("Tenter d'améliorer les %1").arg(ligne+1));
			setAction(toutSauf(ligne+1));
			return true;
		}
	}

	return false;
}

/*************** Garder les meilleurs dés si on est entre la paire et le full, ou le Yahtzee, Ou la chance ************/
bool Analyse::tenterGarderMeilleurs() {
	if (lignesJouables.contains(UNE_PAIRE) || lignesJouables.contains(DEUX_PAIRES) || lignesJouables.contains(BRELAN) || lignesJouables.contains(CARRE) || lignesJouables.contains(CHANCE)) {
		setAction( toutSauf(QList<int>(), 4) );
		return true;
	}
	return false;
}

bool Analyse::tenterFaireSuite() {
	/*************** Tenter les suites ************/
	if (lignesJouables.contains(PETITE_SUITE) || lignesJouables.contains(GRANDE_SUITE)) {
//		trace (tr("Supprimer les doublons pour tenter une suite"));
		setAction (garder(QList<int>() << 2 << 3 << 4 << 5));
		return true;
	}
	return false;
}

void Analyse::marquer()
{
	QList<int> possibleList = getPossible();

	QList<int> listeDispo(enCoursLibre);
	if (enCoursDescendante <= YAHTZEE ) {
		listeDispo << enCoursDescendante;
	}

	if (enCoursMontante >= UN) {
		listeDispo << enCoursMontante;
	}

	QList<int> listeDispoHaut, listeDispoBas;
	foreach (int ligne, listeDispo) {
		if (ligne <= SIX) listeDispoHaut << ligne; else listeDispoBas << ligne;
	}

	// voir si on peut marquer le haut direct

	if (getCount(enCoursDescendante) >= 3) {
		setAction(enCoursDescendante);
		return;
	}

// 	if (getCount(enCoursMontante) >= 3) {
// 		marquerHautMontante = true;
// 		setAction(enCoursMontante);
// 		return;
// 	}

	if (enCoursMontante == CHANCE) {
		setAction(enCoursMontante);
		return;
	}

	int max = 1;
	int ligneCandidateBas = -1;
	foreach (int ligne, listeDispoBas) {
		if (getPoints(ligne) >= max) {
			max = getPoints(ligne);
			ligneCandidateBas = ligne;
		}
	}
	// ici on a le max du bas

/*	if (ligneCandidateBas != -1 && max >= 20) {
		setAction(ligneCandidateBas);
		return;
	}
*/
	int ligneCandidateHaut = -1;
	foreach (int ligne, listeDispoHaut) {
		if (getCount(ligne) >= 3) {
			ligneCandidateHaut = ligne;
		}
	}
	// ici on a le max du haut

	if (ligneCandidateHaut != -1) {
		setAction(ligneCandidateHaut);
		return;
	}

	if (ligneCandidateBas != -1) {
		if (getPoints(ligneCandidateBas) <= 15 && enCoursMontante == YAHTZEE) {
			ra.setAction(YAHTZEE, COLONNE_MONTANTE);
			return;
		}
		else {
			setAction(ligneCandidateBas);
			return;
		}
	}

	// revoir le haut : peut-on marquer sans foirer la colonne ?
	for (int ligne = SIX ; ligne >= UN ; ligne--) {
		if (listeDispoHaut.contains(ligne)) {
			int countLigne = getCount(ligne);
			int manque = (ligne+1) * (3-countLigne);
			if (ligne == enCoursDescendante && m_ecartDescendante >= manque) {
				ra.setAction(ligne, COLONNE_DESCENDANTE);
				return;
			}

			if (ligne == enCoursMontante && m_ecartMontante >= manque) {
				ra.setAction(ligne, COLONNE_MONTANTE);
				return;
			}

			if (enCoursLibre.contains(ligne) && m_ecartLibre >= manque) {
				ra.setAction(ligne, COLONNE_LIBRE);
				return;
			}
		}
	}

	if (enCoursMontante == YAHTZEE) {
		// débloquer la colonne MONTANTE
		setAction(YAHTZEE);
		return;
	}

	if (enCoursDescendante == UN && getPoints(UN) > 0) {
		// débloquer la colonne DESCENDANTE
		setAction(UN);
		return;
	}

//	foreach (int ligne, m_ordrePourBarrer ) {
//		if (listeDispo.contains(ligne) && getPoints(ligne)) {
//			setAction(ligne);
//			return;
//		}
//	}

//	trace (tr("JE VAIS SACRIFIER UNE CASE"));

	// quelle ligne barrer ???
	// en premier, le YAHTZEE de la montante
	if (enCoursMontante == YAHTZEE ) {
		ra.setAction(YAHTZEE, COLONNE_MONTANTE);
		return;
	}

	// ensuite ...

	foreach (int ligne, m_ordrePourBarrer ) {
		if (listeDispo.contains(ligne)) {
			setAction(ligne);
			return;
		}
	}
}


int Analyse::mini( QList<int> liste )
{
	if (liste.isEmpty()) return -1;
	if (liste.count() == 1) return liste[0];

	int min = liste[0];
	foreach (int element, liste) {
		if (element < min) min = element;
	}
	return min;
}


int Analyse::maxi( QList<int> liste )
{
	if (liste.isEmpty()) return -1;
	if (liste.count() == 1) return liste[0];

	int max = liste[0];
	foreach (int element, liste) {
		if (element > max) max = element;
	}
	return max;
}


QList<int> Analyse::toutSauf(Figure listeAGarder, int valMin /* 7 par défaut : tout rejouer */)
{
	QList<int> result;

	for (int numDice = UN ; numDice <= CINQ ; numDice++) {
		if (!listeAGarder.contains(numDice) && (dicesList[numDice] < valMin)) {
			result << numDice;
		}
	}

	return result;
}


QList<int> Analyse::toutSauf(QList<int> listeAGarder, int valMin /* 7 par défaut : tout rejouer */)
{
	QList<int> result;

	for (int numDice = UN ; numDice <= CINQ ; numDice++) {
		if (!listeAGarder.contains(numDice) && (dicesList[numDice] < valMin)) {
			result << numDice;
		}
	}

	return result;
}


QList<int> Analyse::toutSauf(int valeur, int valMin /* 7 par défaut : tout rejouer */)
{
	QList<int> result;

	for (int numDice = UN ; numDice <= CINQ ; numDice++) {
		if (dicesList[numDice] != valeur&& (dicesList[numDice] < valMin)) {
			result << numDice;
		}
	}

	return result;
}


QList<int> Analyse::garder(QList<int> valeurs) {
	QList<int> result;
	QList<int> listeValeurs;

	for (int numDice = UN ; numDice <= CINQ ; numDice++)
	{
		int diceValue = dicesList[numDice];
		if (valeurs.contains(diceValue) && ! listeValeurs.contains(diceValue))
		{
			listeValeurs << diceValue;
			result << numDice;
		}
	}

	return result;
}


QList<int> Analyse::listeDesDontValeurEst(int valeur)
{
	QList<int> result;

	for (int numDice = UN ; numDice <= CINQ ; numDice++)
	{
		if (dicesList[numDice] == valeur) {
			result << numDice;
		}
	}

	return result;
}


int Analyse::maxCountHaut()
{
	int max = qMax(getCount(UN), getCount(DEUX));
	for (int numDice = TROIS ; numDice <= CINQ ; numDice++)
	{
		max = qMax(max, getCount(numDice));
	}

	return max;
}


QList<int> Analyse::lignesMaxCountHaut()
{
	int ligneMax = UN;
	int max = getCount(UN);
	for (int ligne = DEUX ; ligne <= SIX ; ligne++)
	{
		if (getCount(ligne) >= max) {
			max = getCount(ligne);
			ligneMax = ligne;
		}
	}

	QList<int> result;
	for (int ligne = SIX ; ligne >= UN ; ligne--)
	{
		if (getCount(ligne) == max) {
			result << ligne;
		}
	}

	return result;
}


int Analyse::ligneMaxPointsBas()
{
	int ligneMax = UNE_PAIRE;
	int max = getPoints(UNE_PAIRE);
	for (int ligne = DEUX_PAIRES ; ligne <= YAHTZEE ; ligne++)
	{
		if (getPoints(ligne) >= max) {
			max = getPoints(ligne);
			ligneMax = ligne;
		}
	}

	return ligneMax;
}

int Analyse::getMostFrequentValue()
{
	int max = getCount(UN);
	int retLigne = UN;
	for (int ligne = DEUX ; ligne <= SIX ; ligne++ )
	{
		int tmp = getCount(ligne);
		if (tmp >= max)
		{
			max = tmp;
			retLigne = ligne;
		}
	}
	return retLigne;
}

void Analyse::getProba(QList<int> lancer, int figure, qreal *lancer1, qreal *lancer2)
{
	int nbr;
	switch (figure) {
		case UN:
		{
			int occurrences = lancer.count(1);
			int nbrCasFavorables = 5-occurrences;
//			int nbrCasPossibles =
		}
		break;

		case DEUX:
		break;

		case TROIS:
		break;

		case QUATRE:
		break;

		case CINQ:
		break;

		case SIX:
		break;

		case UNE_PAIRE:
		break;

		case DEUX_PAIRES:
		break;

		case BRELAN:
		break;

		case CARRE:
		break;

		case FULL:
		break;

		case PETITE_SUITE:
		break;

		case GRANDE_SUITE:
		break;

		case CHANCE:
		break;

		case YAHTZEE:
		break;
	}
}


void Analyse::getAllProba(QList<int> lancer, QList<qreal> *lancer1, QList<qreal> *lancer2)
{
	// calcule les meilleures probabilités
	lancer1->clear();
	lancer2->clear();

	foreach (int ligne, lignesCliquables) {
		qreal probaLancer1 = 0;
		qreal probaLancer2 = 0;
		getProba(lancer, ligne, &probaLancer1, &probaLancer2);
		*lancer1 << probaLancer1;
		*lancer2 << probaLancer2;
	}

}

