/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef ANALYSE_H
#define ANALYSE_H

#include "includes.h"
#include <QList>
#include <QStringList>
#include <QtDebug>
#include "figure.h"


#include "resultatanalyse.h"

class Analyse : public QObject
{
// objet de la classe : décompte des points et extraction des figures pour préparer l'Analyse proprement dite


	Q_OBJECT

public:
	Analyse ();
	~Analyse () {}
	void init(int joueur, int numSession, int ligneDescendante, int ligneMontante, QList<int> &lignesLibres, int d1, int d2, int d3, int d4, int d5, int ecartDesc, int ecartAsc, int ecartLibre);
	void setSuggestions();
	QList<int> getPossible();
	ResultatAnalyse getResultatAnalyse() { return ra; }
	QString getLabel( int ligne) { return label[ligne]; }

	int valeurFigure( int figure );
	int getCount( int figure ){ return (figure >= UN && figure <= SIX) ? (resultat[figure][NBR]) : -1;}
	int getPoints( int figure ) { return resultat[figure][POINTS]; }
	void setPoints( int figure, int pts ) { resultat[figure][POINTS] = pts; }
	void setAction( int ligne );
	void setAction( QList<int> liste ) { ra.setAction(liste); }

private:
	void trace(QString txt);
	ResultatAnalyse ra;
	QStringList label;
	QList<int> dicesList;
	int resultat[DERNIERE_LIGNE + 1][2];
	int enCoursDescendante;
	int enCoursMontante;
	QList<int> enCoursLibre;
	QList<int> lignesJouables;
	QList<int> enCoursLibreHaut;
	QList<int> enCoursLibreBas;
	int numLancer;
	int m_ecartDescendante;
	int m_ecartMontante;
	int m_ecartLibre;
	int m_avancementDescendante;
	int m_avancementMontante;
	int m_joueur;
	QList<int> m_ordrePourBarrer;

	Figure yahtzee;
	Figure carre;
	Figure brelan;
	Figure full;
	Figure paire1;
	Figure paire2;
	Figure deuxPaires;
	Figure petiteSuite;
	Figure grandeSuite;

	int sommeDes()
		{ return dicesList[UN] + dicesList[DEUX] + dicesList[TROIS] + dicesList[QUATRE] + dicesList[CINQ]; }
	int maxCountHaut();
	int ligneMaxPointsBas();
	QList<int> lignesMaxCountHaut();
	QList<int> lignesCliquables;

	void marquer(); // c'est le dernier tour, il faut jouer
	bool tenterFigureNonAmeliorable();

	void tenterAmeliorer();
	bool tenterPetiteSuite();
	bool tenterDescendante();
	bool tenterMontante();
	bool tenterCarre();
	bool tenterBrelan();
	bool tenterDeuxPaires();
	bool tenterPaire();
	bool tenterChance();
	bool tenterHaut2();
	bool tenterGarderMeilleurs();
	bool tenterFaireSuite();

	QList<int> toutSauf(Figure listeAGarder, int valMin = 7);
	QList<int> toutSauf(QList<int> listeAGarder, int valMin = 7);
	QList<int> toutSauf(int valeur, int valMin = 7);
	QList<int> garder(QList<int>);
	QList<int> listeDesDontValeurEst(int val);
	int getMostFrequentValue();
	int mini( QList<int> );
	int maxi( QList<int> );

	void getProba(QList<int> lancer, int figure, qreal *lancer1, qreal *lancer2);
	void getAllProba(QList<int> lancer, QList<qreal> *lancer1, QList<qreal> *lancer2);

};


#endif
