/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#include "piste.h"

//#define DOUBLE_BUFFERING

// couleur du tapis : 0x247935
#define TAPIS			0x247935
#define BORDERSIZE		4.0
#define RAYON			12
#define LIGHTER_DARKER	120


Piste::Piste (QWidget *parent) : QWidget(parent)
{
	m_backgroundColor = QColor(TAPIS);
	setAutoFillBackground(false);
}


void Piste::paintEvent (QPaintEvent *)
{

#ifdef DOUBLE_BUFFERING
	QImage image(size(), QImage::Format_ARGB32_Premultiplied);
	image.fill(0);	// fond transparent
	QPainter painter(&image);
	redraw (painter);
	QPainter widgetPainter(this);
	widgetPainter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform, true);
	widgetPainter.drawImage(0, 0, image);
#else
	QPainter painter(this);
	redraw (painter);
#endif
}

void Piste::redraw(QPainter& painter)
{

	painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform, true);
	QPen pen(QBrush(Qt::black), BORDERSIZE);
	painter.setPen(pen);

	QLinearGradient gradient(rect().topLeft(), rect().bottomRight());
	gradient.setColorAt(0.0, m_backgroundColor.lighter(LIGHTER_DARKER));
	gradient.setColorAt(1.0, m_backgroundColor.darker(LIGHTER_DARKER));
	painter.setBrush(gradient);

	QRect surface(rect().adjusted(BORDERSIZE/2, BORDERSIZE/2, -BORDERSIZE/2, -BORDERSIZE/2));
	painter.drawRoundedRect(surface, RAYON, RAYON);
}

