/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//#define DOUBLE_BUFFERING


#define DICE_SIZE			63
#define SIZE_WHEN_SELECTED	70
#define FULL_SIZE			100
#define SELECT_DURATION		200
#define DARKER_LIGHTER		200
#define ROLL_ANIM_DURATION	1000
#define	TOUR_STEPS			360
#define LOGO				":/images/distrologo.png"

#include "renderarea.h"
#include <QtGui>

DiceRenderArea::DiceRenderArea(QWidget *parent) : QWidget(parent)
{
	setMinimumSize (QSize(DICE_SIZE, DICE_SIZE));
	setMaximumSize (QSize(DICE_SIZE, DICE_SIZE));

	m_tlAnim = new QTimeLine(ROLL_ANIM_DURATION, this);
	m_tlSize = new QTimeLine(ROLL_ANIM_DURATION, this);

	connect(m_tlAnim, SIGNAL(frameChanged(int)), this, SLOT(anim(int)));
	connect(m_tlSize, SIGNAL(frameChanged(int)), this, SLOT(changeScale()));
	connect(m_tlSize, SIGNAL(finished()), m_tlSize, SLOT(stop()));

	m_checked = false;
	m_animation = false;
	m_enabled = true;
	m_diceValue = 6;	// arbitraire
	m_aspect = CLASSIQUE;	// points affichés

	m_selectionColour = VERT_CLAIR;	// couleur de sélection
	setCaro(0);	// easter egg
	setLogo( true );
	m_fast = false;
	m_scaleXY = 1.0;

	// liste des points à dessiner pour chaque valeur du dé
	m_pointNumList[0] << 0 << 1 << 2 << 3 << 4 << 5 << 6 << 7 << 8;
	m_pointNumList[1] << 4;
	m_pointNumList[2] << 2 << 6;
	m_pointNumList[3] << 2 << 4 << 6;
	m_pointNumList[4] << 0 << 2 << 6 << 8;
	m_pointNumList[5] << 0 << 2 << 4 << 6 << 8;
	m_pointNumList[6] << 0 << 2 << 3 << 5 << 6 << 8;
	m_pointNumList[7] << 0 << 2 << 4 << 6 << 7 << 8;

}

DiceRenderArea::~DiceRenderArea ()
{
	delete m_tlAnim;
	delete m_tlSize;
}


void DiceRenderArea::setFast(bool newFast)
{
	if (m_fast != newFast) {
		m_fast = newFast;
		update();
	}
}

void DiceRenderArea::setLogo (bool drawLogo)
{
	m_logo = drawLogo;
	m_icon = drawLogo ? QIcon(LOGO) : QIcon();
}

void DiceRenderArea::clearSeed()
{
	m_lancer[HUMAN].clear();
	m_lancer[PC].clear();
}

void DiceRenderArea::setDiceValue (int id, int value)
{
	// donne une valeur entre 1 et 6 au dé pour l'utilisateur id.
	// ensuite redessine le dé
	// Pour une valeur aléatoire : setDiceValue(-1) ou setDiceValue()

	if (value != -1) {
		m_diceValue = value;
		update();
	}
	else {
		if (m_lancer[HUMAN].isEmpty() || m_lancer[PC].isEmpty()) {
			int valeur = alea1_6();
			m_lancer[HUMAN].enqueue(valeur);
			m_lancer[PC].enqueue(valeur);
		}
		m_diceValue = m_lancer[id].dequeue();
		update();
	}
}


void DiceRenderArea::setAnimation (bool anim)
{
	// anime ou immobilise le dé
	m_animation = anim;
	m_enabled = !anim;

	if (m_animation){
		m_sensAnim = (alea(100) > 50 ? -1 : 1);

//		m_tlAnim->setCurveShape(QTimeLine::EaseInCurve);
//		m_tlAnim->setCurveShape(QTimeLine::EaseOutCurve);
		m_tlAnim->setCurveShape(QTimeLine::EaseInOutCurve);
//		m_tlAnim->setCurveShape(QTimeLine::LinearCurve);
//		m_tlAnim->setCurveShape(QTimeLine::SineCurve);
//		m_tlAnim->setCurveShape(QTimeLine::CosineCurve);

		m_tlAnim->setFrameRange(0, TOUR_STEPS);
		m_tlAnim->setDuration(ROLL_ANIM_DURATION);
		m_tlAnim->setLoopCount(0);
		m_tlAnim->start();
	}
	else {
		m_tlAnim->stop();
	}
}

void DiceRenderArea::anim(int frame)
{
	m_diceValue = alea1_6();
	m_angleAnim = frame;
	update();
}


void DiceRenderArea::paintEvent(QPaintEvent *)
{

#ifdef DOUBLE_BUFFERING
	QImage image(size(), QImage::Format_ARGB32_Premultiplied);
	image.fill(0);	// fond transparent
	QPainter painter(&image);
	redraw (painter);
	QPainter widgetPainter(this);
	widgetPainter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform | QPainter::HighQualityAntialiasing, true);
	widgetPainter.drawImage(0, 0, image);
#else
	QPainter painter(this);
	redraw (painter);
#endif
}

void DiceRenderArea::redraw (QPainter &painter)
{
	painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform | QPainter::HighQualityAntialiasing, true);

	int translateX = rect().center().x();
	int translateY = rect().center().y();

	painter.translate(translateX, translateY);

	if (m_animation) painter.rotate(m_sensAnim * 1 * m_angleAnim);

	painter.scale(m_scaleXY, m_scaleXY);

	painter.translate(-translateX, -translateY);

	QColor faceColour(m_checked ? m_selectionColour: Qt::white);

	if (!m_fast) {
		QLinearGradient linearGradient (rect().topLeft(), rect().bottomRight());
		linearGradient.setColorAt(0.0, faceColour);
		linearGradient.setColorAt(1.0, faceColour.darker(DARKER_LIGHTER));
		painter.setBrush (QBrush(linearGradient));
	}
	else {
		painter.setBrush (QBrush(faceColour.darker(125)));
	}

	painter.setPen (Qt::NoPen);
	painter.drawRoundedRect(rect(), 20.0, 20.0, Qt::RelativeSize);

	if (m_diceValue < 0 || m_diceValue > 7 ) return;

	QRectF rectF(rect());

	if (m_aspect == CHIFFRES || m_animation) {

		QFont font;
		font.setPixelSize(height() / 2);
		font.setBold(true);
		painter.setFont(font);
		painter.setPen (Qt::black);
		if (m_animation) {
			painter.drawText(rectF, Qt::AlignCenter, "?");
		}
		else {
			if (m_logo) m_icon.paint(&painter, rect());
			painter.drawText(rectF, Qt::AlignCenter, QString("%1").arg(m_diceValue));
		}
		return;
	}

	if (m_aspect == CLASSIQUE) {

		qreal xGauche = width() / 4.0;
		qreal yHaut = height() / 4.0;

		qreal xCoord[9] = {xGauche, 2 * xGauche, 3 * xGauche, xGauche, 2 * xGauche, 3 * xGauche, xGauche, 2 * xGauche, 3 * xGauche};
		qreal yCoord[9] = {yHaut, yHaut, yHaut, 2 * yHaut, 2 * yHaut, 2 * yHaut, 3 * yHaut, 3 * yHaut, 3 * yHaut};

		qreal radius = qMin(width(), height()) / 9.0;
		qreal cote = radius * 2.0;

		// fabrication de l'image du point.
		QImage imagePoint(QSize(cote, cote), QImage::Format_ARGB32_Premultiplied);
		imagePoint.fill(0);	// fond transparent
		QPainter pointPainter(&imagePoint);
		pointPainter.setRenderHints(QPainter::Antialiasing, true);
		QLinearGradient pointGradient(0, 0, cote, cote);
		pointGradient.setColorAt(0.1, Qt::black);
		pointGradient.setColorAt(0.8, faceColour);
		pointPainter.setBrush(QBrush(pointGradient));
		pointPainter.setPen (Qt::NoPen);
		pointPainter.drawEllipse(0, 0, cote, cote);
		if (m_logo) m_icon.paint(&pointPainter, QRect(0, 0, cote, cote));

		painter.setPen (Qt::NoPen);

		foreach (int numPoint, m_pointNumList[m_diceValue]) {
			// x et y sont les coordonnées haut-gauche du carré dans lequel on va dessiner le creux du dé

			qreal x = xCoord[numPoint] - radius;
			qreal y = yCoord[numPoint] - radius;

			QRect carre(x, y, cote, cote);

			switch (caro) {
				case 0:	// chaque creux a un aspect 'normal'
				{
					if (!m_fast) {	// on redessine l'image du point
						painter.drawImage(x, y, imagePoint);
					}
					else {
						painter.setBrush(Qt::black);
						painter.drawEllipse(carre);
						if (m_logo) m_icon.paint(&painter, carre);
					}
				}
				break;

				case 1:	// chaque creux affiche la valeur du dé
				case 2:	// chaque creux affiche les initiales de...
				{
					QFont font;
					font.setPixelSize(cote);
					font.setBold(true);
					painter.setFont (font);
					painter.setPen (QPen(Qt::black));
					switch (caro) {
						case 1:
							painter.drawText(carre, Qt::AlignCenter, QString("%1").arg(m_diceValue));
							break;

						case 2:
							painter.drawText(carre, Qt::AlignCenter, m_txtCaro);
							break;
					}
				}
				break;
			}
		}
	}
}

void DiceRenderArea::mouseReleaseEvent (QMouseEvent *event)
{
	if (isEnabled())
	{
		if (event->button() == Qt::LeftButton) setChecked (!m_checked);

		if (event->button() == Qt::RightButton)
		{
			caro = (caro + 1) % 3;
			update();
		}
	}
}


void DiceRenderArea::setChecked (bool newCheck)
{
	if ( m_checked != newCheck )
	{
		if (m_tlSize->state() == QTimeLine::Running) {	// arrêt de la QTimeLine si en cours
			m_tlSize->stop();
		}

		m_checked = newCheck;
		if (m_checked) {
			m_tlSize->setFrameRange(FULL_SIZE, SIZE_WHEN_SELECTED);
		}
		else {
			m_tlSize->setFrameRange(SIZE_WHEN_SELECTED, FULL_SIZE);
		}
//		m_tlSize->setCurveShape(QTimeLine::EaseInCurve);
		m_tlSize->setCurveShape(QTimeLine::EaseOutCurve);
//		m_tlSize->setCurveShape(QTimeLine::EaseInOutCurve);
//		m_tlSize->setCurveShape(QTimeLine::LinearCurve);
//		m_tlSize->setCurveShape(QTimeLine::SineCurve);
//		m_tlSize->setCurveShape(QTimeLine::CosineCurve);

		m_tlSize->setDuration(SELECT_DURATION);
		m_tlSize->setLoopCount(1);

		m_tlSize->start();

		emit diceChecked();
	}
}


void DiceRenderArea::changeScale() {
	m_scaleXY = m_tlSize->currentFrame()/100.0;
	update();
}



void DiceRenderArea::setSelectionColour(QColor value)
{
	m_selectionColour = value;
	update();
}


void DiceRenderArea::setSurfaceAspect(int value)
{
	m_aspect = value;
	update();
}


void DiceRenderArea::setEnabled(bool enable)
{
	m_enabled = enable;
}

bool DiceRenderArea::isEnabled()
{
	return m_enabled;
}
