/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef RENDERAREA_H
#define RENDERAREA_H

//#include "ui_yahtzee.h"
#include "includes.h"

// #include <QSize>
// #include <QTimer>

//#include <QtGui>
#include <QWidget>
#include <QPainter>
#include <QColor>
#include <QString>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QQueue>
#include <QList>
#include <QIcon>
#include <QTimeLine>

#define CLASSIQUE		0
#define CHIFFRES		1
#define ROUGE_CLAIR		0xFFB7B6
#define JAUNE_CLAIR		0xF8FFB6
#define VERT_CLAIR		0xDDFFB6
#define BLEU_CLAIR		0xBBF7FF


class DiceRenderArea : public QWidget
{
	Q_OBJECT

	public:
		DiceRenderArea (QWidget *parent = 0);
		~DiceRenderArea ();

		void redraw (QPainter&);
		void setDiceValue (int id, int value = -1);
		int getDiceValue () { return m_diceValue; }
		void setChecked (bool);
		bool isChecked () { return m_checked; }
		void setAnimation (bool);
		void setSelectionColour(QColor value);
		void setSurfaceAspect(int value);
		void setCaro(int value, QString strValue = "cl") { caro = value; m_txtCaro = strValue.left(2); }
		void setLogo( bool drawLogo );
		void setEnabled(bool);
		bool isEnabled();
		void clearSeed();

		void paintEvent (QPaintEvent *event);
		void mouseReleaseEvent (QMouseEvent *event);

		static int alea1_6 () { return alea(6); }
		static int alea(int n) { return rand() % n + 1; }

	public slots:
		void setFast(bool newFast);

	private:
		QQueue<int> m_lancer[2];
		QList<int> m_pointNumList[8];
		qreal m_scaleXY;
		int m_sensAnim;
		bool m_logo;
		QIcon m_icon;
		bool m_fast;
		QColor m_selectionColour;
		int m_aspect;
		int m_diceValue;
		bool m_checked;
		bool m_enabled;

		int caro;

		QString m_txtCaro;
		bool m_animation;
		int m_angleAnim;
		QTimeLine *m_tlAnim;
		QTimeLine *m_tlSize;

	signals:
		void diceChecked();

	private slots:
		void anim(int);
		void changeScale();
};

#endif
